/**********************************************************
 * BWave Video Codec
 * coding
 * Copyricht (c) 2004 by Thomas Bergwinkl
 **********************************************************/

#include "../global.h"


CVECTOR cvector[1024*1024];


void encode_cvector(BITSTREAM *bits, CVECTOR *cvector, int vectors) {
	int i;
	
	encode_uiv(bits, vectors);
	
	for(i=0; i<vectors; i++) {
		encode_uiv(bits, cvector[i].run);
		encode_iv(bits, cvector[i].value);
	}
}


int decode_cvector(BITSTREAM *bits, CVECTOR *cvector) {
	int vectors, i;
	
	vectors = decode_uiv(bits);
	
	for(i=0; i<vectors; i++) {
		cvector[i].run = decode_uiv(bits);
		cvector[i].value = decode_iv(bits);
	}

	return vectors;
}


void encode_plane(BITSTREAM *bits, int16_t *data, int width, int height,
	int stride) {

	int x, y;

	for(y=0; y<height; y++) {
		for(x=0; x<width; x++)
			encode_iv(bits, data[x]);
		data += stride;
	}
}


/*void debug_cv(CVECTOR *cvector, int vectors, int width, int height) {
	int i, r=0;
	
	for(i=0; i<vectors; i++)
		r += cvector[i].run+1;
	
	if(r > width*height)
		printf("%d %d\n", r, width*height);
}*/


void encode_plane_cv(BITSTREAM *bits, int16_t *data, int width, int height,
	int stride) {

	int x, y, r=0, vectors=0;
	
	for(y=0; y<height; y++) {
		for(x=0; x<width; x++)
			if(data[x]) {
				int t = y*width + x;
				cvector[vectors].run = t-r;
				cvector[vectors].value = data[x];
				r = t+1;
				vectors++;
			}
		data += stride;
	}
	encode_cvector(bits, cvector, vectors);
}


void decode_plane(BITSTREAM *bits, int16_t *data, int width, int height,
	int stride) {

	int x, y;
	
	for(y=0; y<height; y++) {
		for(x=0; x<width; x++)
			data[x] = decode_iv(bits);
		data += stride;
	}
}


void decode_plane_cv(BITSTREAM *bits, int16_t *data, int width, int height,
	int stride) {

	int i, r=0, vectors;
	
	vectors = decode_cvector(bits, cvector);

	for(i=0; i<vectors; i++) {
		r += cvector[i].run;
		if(r < width*height) /*todo*/
			data[(r/width)*stride + (r%width)] = cvector[i].value;
		r++;
	}
}


void encode_wl_level(BITSTREAM *bits, WL_LEVEL *wl_level, int levels,
	int stride) {

	int i;

	encode_plane(bits, wl_level[levels-1].ll, wl_level[levels-1].width_l,
		wl_level[levels-1].height_l, stride);

	for(i=0; i<levels; i++) {
		encode_uiv(bits, wl_level[i].quant);

		encode_plane_cv(bits, wl_level[i].hl, wl_level[i].width_h,
			wl_level[i].height_l, stride);

		encode_plane_cv(bits, wl_level[i].lh, wl_level[i].width_l,
			wl_level[i].height_h, stride);

		encode_plane_cv(bits, wl_level[i].hh, wl_level[i].width_h,
			wl_level[i].height_h, stride);
	}
}


void decode_wl_level(BITSTREAM *bits, WL_LEVEL *wl_level, int levels,
	int stride) {

	int i;

	decode_plane(bits, wl_level[levels-1].ll, wl_level[levels-1].width_l,
		wl_level[levels-1].height_l, stride);
	
	for(i=0; i<levels; i++) {
		wl_level[i].quant = decode_uiv(bits);

		decode_plane_cv(bits, wl_level[i].hl, wl_level[i].width_h,
			wl_level[i].height_l, stride);

		decode_plane_cv(bits, wl_level[i].lh, wl_level[i].width_l,
			wl_level[i].height_h, stride);

		decode_plane_cv(bits, wl_level[i].hh, wl_level[i].width_h,
			wl_level[i].height_h, stride);
	}
}


void encode_wl_image(BITSTREAM *bits, WL_IMAGE *wl_image) {
	encode_wl_level(bits, wl_image->wl_level_y, wl_image->levels,
 		wl_image->width);

	encode_wl_level(bits, wl_image->wl_level_u, wl_image->levels,
 		wl_image->width/2);

	encode_wl_level(bits, wl_image->wl_level_v, wl_image->levels,
 		wl_image->width/2);
}


void decode_wl_image(BITSTREAM *bits, WL_IMAGE *wl_image) {
	memset(wl_image->y, 0, wl_image->width*wl_image->height*sizeof(int16_t));
	decode_wl_level(bits, wl_image->wl_level_y, wl_image->levels,
 		wl_image->width);

	memset(wl_image->u, 0,
 		(wl_image->width*wl_image->height*sizeof(int16_t))/4);
	decode_wl_level(bits, wl_image->wl_level_u, wl_image->levels,
 		wl_image->width/2);

	memset(wl_image->v, 0,
		(wl_image->width*wl_image->height*sizeof(int16_t))/4);
	decode_wl_level(bits, wl_image->wl_level_v, wl_image->levels,
 		wl_image->width/2);
}


