/**********************************************************
 * BWave Video Codec
 * image management
 * Copyricht (c) 2004 by Thomas Bergwinkl
 **********************************************************/

#include "../global.h"



void image_create(IMAGE *image, int width, int height) {
	image->width = width;
	image->height = height;
	image->stride_y = ((width+15)>>4)<<4;
	image->stride_uv = image->stride_y/2;
	image->y = (uint8_t*)bw_malloc(image->stride_y*image->height*sizeof(uint8_t));
	image->u = (uint8_t*)bw_malloc(image->stride_uv*(image->height/2)*sizeof(uint8_t));
	image->v = (uint8_t*)bw_malloc(image->stride_uv*(image->height/2)*sizeof(uint8_t));
}


void image_destroy(IMAGE *image) {
	image->width = 0;
	image->height = 0;
	image->stride_y = 0;
	image->stride_uv = 0;
	bw_free(image->y);
	bw_free(image->u);
	bw_free(image->v);
}


static void
safe_packed_conv(uint8_t * x_ptr, int x_stride,
				 uint8_t * y_ptr, uint8_t * u_ptr, uint8_t * v_ptr,
				 int y_stride, int uv_stride,
				 int width, int height, int vflip,
				 packedFunc * func_opt, packedFunc func_c, int size)
{
	int width_opt, width_c;

	if (func_opt != func_c && x_stride < size*((width+15)/16)*16)
	{
		width_opt = width & (~15);
		width_c = width - width_opt;
	}
	else
	{
		width_opt = width;
		width_c = 0;
	}

	func_opt(x_ptr, x_stride,
			y_ptr, u_ptr, v_ptr, y_stride, uv_stride,
			width_opt, height, vflip);

	if (width_c)
	{
		func_c(x_ptr + size*width_opt, x_stride,
			y_ptr + width_opt, u_ptr + width_opt/2, v_ptr + width_opt/2,
			y_stride, uv_stride, width_c, height, vflip);
	}
}


void image_import(IMAGE *image, void *data, int stride, int colorspace) {
	switch(colorspace) {
		case BW_COL_RGB24:
			safe_packed_conv(data, stride, image->y, image->u, image->v,
				image->stride_y, image->stride_uv, image->width, image->height, 1,
				bgr_to_yv12, bgr_to_yv12_c, 3);
			break;

		case BW_COL_YV12:
			yv12_to_yv12(image->y, image->v, image->u, image->stride_y,
				image->stride_uv, data, data+stride*image->height,
				data+stride*image->height+(stride/2)*(image->height/2),
				stride, stride/2, image->width, image->height, 0);
			break;
	}
}


void image_export(IMAGE *image, void *data, int stride, int colorspace) {
	switch(colorspace) {
		case BW_COL_RGB24:
			safe_packed_conv(data, stride, image->y, image->u, image->v,
				image->stride_y, image->stride_uv, image->width, image->height, 1,
				yv12_to_bgr, yv12_to_bgr_c, 3);
			break;

		case BW_COL_YV12:
			yv12_to_yv12(data,  data+stride*image->height,
				data+stride*image->height+(stride/2)*(image->height/2), stride,
				stride/2, image->y, image->v, image->u, image->stride_y,
				image->stride_uv, image->width, image->height, 0);
			break;
	}
}

