/**********************************************************
 * BWave Video Codec
 * wavelet transform low level
 * Copyricht (c) 2004 by Thomas Bergwinkl
 **********************************************************/

#include "../global.h"


void fwt_cpy(int16_t *x, int16_t *d, int stride, int size) {
	int i, j=size-(size/2);
	
	x += j*stride;
	
	for(i=j; i<size; i++) {
		x[0] = d[0];
		x+=stride;
		d++;
	}
}


void fwt_ana(int16_t *x, int16_t *d, int stride, int size) {
	int i, k=size/2;
	int16_t *p=x;
	
	if(size&1) {
		for(i=0; i<k; i++) {
			d[i] = p[stride] - (p[0] + p[stride*2]) / 2;
			p+=stride*2;
		}
	} else {
		for(i=0; i<k-1; i++) {
			d[i] = p[stride] - (p[0] + p[stride*2]) / 2;
			p+=stride*2;
		}
		d[k-1] = x[(size-1)*stride] - x[(size-2)*stride];
	}
}


void fwt_syn(int16_t *x, int16_t *s, int16_t *d, int stride, int size) {
	int i, k=size/2;
	int16_t *p=x+stride*2, *q=s+stride; 
	
	s[0] = x[0] + d[1] / 2;
	
	for(i=1; i<k; i++) {
		q[0] = p[0] + (d[i-1] + d[i]) / 4;
		p+=stride*2;
		q+=stride;
	}
	
	if(size&1)
		s[k*stride] = x[2*k*stride] + d[k-1] / 2;
}


void fwt_line(int16_t *data, int16_t *buf, int stride, int size) {
	int16_t *x = data;
	int16_t *d = buf;
	int16_t *s = data;

	fwt_ana(x, d, stride, size);
	fwt_syn(x, s, d, stride, size);
	fwt_cpy(x, d, stride, size);
}



void fwt_block(int16_t *data, int16_t *buf, int line, int size, int stride) {
	int16_t *tmp;
	int tline=line, tsize=size;
	
	tmp = data;
	for(;tline;tline--, tmp+=stride)
		fwt_line(tmp, buf, 1, size);
		
	tmp = data;
	for(;tsize;tsize--, tmp++)
		fwt_line(tmp, buf, stride, line);
}




void iwt_cpy(int16_t *x, int16_t *d, int stride, int size) {
	int i;
	
	for(i=0; i<size; i++) {
		d[i] = x[0];
		x+=stride;
	}
}


void iwt_ana(int16_t *x, int16_t *d, int stride, int size) {
	int i, k=size/2;
	int16_t *p=x;
	
	if(size&1) {
		for(i=0; i<k; i++) {
			p[stride] = d[i] + (p[0] + p[stride*2]) / 2;
			p+=stride*2;
		}
	} else {
		for(i=0; i<k-1; i++) {
			p[stride] = d[i] + (p[0] + p[stride*2]) / 2;
			p+=stride*2;
		}
		x[(size-1)*stride] = d[k-1] + x[(size-2)*stride];
	}
}


void iwt_syn(int16_t *x, int16_t *s, int16_t *d, int stride, int size) {
	int i, k=size/2;
	int16_t *p=x+stride*2;
	
	x[0] = s[0] - d[1] / 2;
	
	for(i=1; i<k; i++) {
		p[0] = s[i] - (d[i-1] + d[i]) / 4;
		p+=stride*2;
	}
	
	if(size&1)
		x[2*k*stride] = s[k] - d[k-1] / 2;
}


void iwt_line(int16_t *data, int16_t *buf, int stride, int size) {
	int16_t *x = data;
	int16_t *s = buf;
	int16_t *d = buf + size - (size/2);

	iwt_cpy(x, s, stride, size);
	iwt_syn(x, s, d, stride, size);
	iwt_ana(x, d, stride, size);
}


void iwt_block(int16_t *data, int16_t *buf, int line, int size, int stride) {
	int16_t *tmp;
	int tline=line, tsize=size;

	tmp=data;
	for(;tsize;tsize--, tmp++)
		iwt_line(tmp, buf, stride, line);

	tmp=data;
	for(;tline;tline--, tmp+=stride)
		iwt_line(tmp, buf, 1, size);
}


