/**********************************************************
 * BWave Video Codec
 * wavelet transform high level
 * Copyricht (c) 2004 by Thomas Bergwinkl
 **********************************************************/

#include "../global.h"


int ilog2(int x) {
	int i=0;

	while((x>>=1))
		i++;
	
	return i;
}


void plane_quant(int16_t *data, int width, int height, int stride, int quant,
	int deadz) {
	int x, y;
	
	for(y=0; y<height; y++) {
		for(x=0; x<width; x++)
			if(ABS(data[x]) < deadz)
				data[x] = 0;
			else
				data[x] = (data[x]+quant/2)/quant;
		data += stride;
	}
}


void plane_dequant(int16_t *data, int width, int height, int stride,
	int quant) {
	int x, y;
	
	for(y=0; y<height; y++) {
		for(x=0; x<width; x++)
			data[x] *= quant;
		data += stride;
	}
}


void wl_level_create(WL_LEVEL *wl_level, int levels, int16_t *data, int width,
	int height, int stride) {
	
	int i;
	
	for(i=0; i<levels; i++) {
		wl_level[i].width = width;
		wl_level[i].height = height;
		wl_level[i].stride = stride;
		wl_level[i].width_l = width/2;
		wl_level[i].height_l = height/2;
		wl_level[i].width_h = width - (width/2);
		wl_level[i].height_h = height - (height/2);
		
		wl_level[i].ll = data;
		wl_level[i].hl = data+wl_level[i].width_l;
		wl_level[i].lh = data+wl_level[i].height_l*stride;
		wl_level[i].hh = data+wl_level[i].height_l*stride+wl_level[i].width_l;
		
		width = wl_level[i].width_l;
		height = wl_level[i].height_l;
	}
}


void wl_level_set_quant(WL_LEVEL *wl_level, int levels, int quant, int deadz) {
	int i;
	
	for(i=0; i<levels; i++) {
		wl_level[i].quant = quant;
		wl_level[i].deadz = deadz;
		quant = (quant+1)/2;
		deadz /= 2;
	}
}


void wl_level_quant(WL_LEVEL *wl_level, int levels, int stride) {
	int i;

	for(i=0; i<levels; i++) {
		plane_quant(wl_level[i].hl, wl_level[i].width_h, wl_level[i].height_l,
			stride, wl_level[i].quant, wl_level[i].deadz);

		plane_quant(wl_level[i].lh, wl_level[i].width_l, wl_level[i].height_h,
			stride, wl_level[i].quant, wl_level[i].deadz);

		plane_quant(wl_level[i].hh, wl_level[i].width_h, wl_level[i].height_h,
			stride, wl_level[i].quant, wl_level[i].deadz);
	}
}


void wl_level_dequant(WL_LEVEL *wl_level, int levels, int stride) {
	int i;
	
	for(i=0; i<levels; i++) {
		plane_dequant(wl_level[i].hl, wl_level[i].width_h, wl_level[i].height_l,
			stride, wl_level[i].quant);

		plane_dequant(wl_level[i].lh, wl_level[i].width_l, wl_level[i].height_h,
			stride, wl_level[i].quant);

		plane_dequant(wl_level[i].hh, wl_level[i].width_h, wl_level[i].height_h,
			stride, wl_level[i].quant);
	}
}


void wl_image_create(WL_IMAGE *wl_image, int width, int height) {
	wl_image->width = width;
	wl_image->height = height;
	wl_image->stride_y = width;
	wl_image->stride_uv = width/2;
	wl_image->y = (int16_t*)bw_malloc(width*height*sizeof(int16_t));
	wl_image->u = (int16_t*)bw_malloc((width*height*sizeof(int16_t))/4);
	wl_image->v = (int16_t*)bw_malloc((width*height*sizeof(int16_t))/4);
	wl_image->levels = MIN(ilog2(width), ilog2(height)) - 3;
	wl_level_create(wl_image->wl_level_y, wl_image->levels, wl_image->y,
 		wl_image->width, wl_image->height, wl_image->stride_y);
	wl_level_create(wl_image->wl_level_u, wl_image->levels, wl_image->u,
 		wl_image->width/2, wl_image->height/2, wl_image->stride_uv);
	wl_level_create(wl_image->wl_level_v, wl_image->levels, wl_image->v,
 		wl_image->width/2, wl_image->height/2, wl_image->stride_uv);
}


void wl_image_set_quant(WL_IMAGE *wl_image, int quant, int deadz) {
	wl_level_set_quant(wl_image->wl_level_y, wl_image->levels, quant, deadz);
	wl_level_set_quant(wl_image->wl_level_u, wl_image->levels, quant, deadz);
	wl_level_set_quant(wl_image->wl_level_v, wl_image->levels, quant, deadz);
}

void wl_image_destroy(WL_IMAGE *wl_image) {
	wl_image->width = 0;
	wl_image->height = 0;
	wl_image->stride_y = 0;
	wl_image->stride_uv = 0;
	bw_free(wl_image->y);
	bw_free(wl_image->u);
	bw_free(wl_image->v);
}

void wl_image_import(WL_IMAGE *wl_image, IMAGE *image) {
	int x, y;
	uint8_t *s;
	int16_t *d;

	s = image->y;
	d = wl_image->y;
	for(y=image->height; y; y--) {
		for(x=image->width; x; x--)
			*(d++) = (int16_t)*(s++);
		s += image->stride_y - image->width;
		d += wl_image->stride_y - wl_image->width;
	}

	s = image->u;
	d = wl_image->u;
	for(y=image->height/2; y; y--) {
		for(x=image->width/2; x; x--)
			*(d++) = (int16_t)*(s++);
		s += image->stride_uv - image->width/2;
		d += wl_image->stride_uv - wl_image->width/2;
	}

	s = image->v;
	d = wl_image->v;
	for(y=image->height/2; y; y--) {
		for(x=image->width/2; x; x--)
			*(d++) = (int16_t)*(s++);
		s += image->stride_uv - image->width/2;
		d += wl_image->stride_uv - wl_image->width/2;
	}
}


void wl_image_export(WL_IMAGE *wl_image, IMAGE *image) {
	int x, y;
	int16_t *s;
	uint8_t *d;
	
	s = wl_image->y;
	d = image->y;
	for(y=image->height; y; y--) {
		for(x=image->width; x; x--) {
			*(d++) = (uint8_t)CLIP(*s, 0, 255);
			s++;
		}
		s += wl_image->stride_y - wl_image->width;
		d += image->stride_y - image->width;
	}

	s = wl_image->u;
	d = image->u;
	for(y=image->height/2; y; y--) {
		for(x=image->width/2; x; x--) {
			*(d++) = (uint8_t)CLIP(*s, 0, 255);
			s++;
		}
		s += wl_image->stride_uv - wl_image->width/2;
		d += image->stride_uv - image->width/2;
	}

	s = wl_image->v;
	d = image->v;
	for(y=image->height/2; y; y--) {
		for(x=image->width/2; x; x--) {
			*(d++) = (uint8_t)CLIP(*s, 0, 255);
			s++;
		}
		s += wl_image->stride_uv - wl_image->width/2;
		d += image->stride_uv - image->width/2;
	}
}


void wl_image_fwt(WL_IMAGE *wl_image) {
	int i;
	int16_t buf[1024];
	
	for(i=0; i<wl_image->levels; i++)
		fwt_block(wl_image->y, buf, wl_image->wl_level_y[i].height,
  			wl_image->wl_level_y[i].width, wl_image->stride_y);
  		
	for(i=0; i<wl_image->levels; i++)
		fwt_block(wl_image->u, buf, wl_image->wl_level_u[i].height,
  			wl_image->wl_level_u[i].width, wl_image->stride_uv);

	for(i=0; i<wl_image->levels; i++)
		fwt_block(wl_image->v, buf, wl_image->wl_level_v[i].height,
  			wl_image->wl_level_v[i].width, wl_image->stride_uv);
  			
	wl_level_quant(wl_image->wl_level_y, wl_image->levels, wl_image->stride_y);
	wl_level_quant(wl_image->wl_level_u, wl_image->levels, wl_image->stride_uv);
	wl_level_quant(wl_image->wl_level_v, wl_image->levels, wl_image->stride_uv);
}


void wl_image_iwt(WL_IMAGE *wl_image) {
	int i;
	int16_t buf[1024];
	
	wl_level_dequant(wl_image->wl_level_y, wl_image->levels, wl_image->stride_y);
	wl_level_dequant(wl_image->wl_level_u, wl_image->levels, wl_image->stride_uv);
	wl_level_dequant(wl_image->wl_level_v, wl_image->levels, wl_image->stride_uv);

	for(i=wl_image->levels-1; i>=0; i--)
		iwt_block(wl_image->y, buf, wl_image->wl_level_y[i].height,
			wl_image->wl_level_y[i].width, wl_image->stride_y);

	for(i=wl_image->levels-1; i>=0; i--)
		iwt_block(wl_image->u, buf, wl_image->wl_level_u[i].height,
			wl_image->wl_level_u[i].width, wl_image->stride_uv);

	for(i=wl_image->levels-1; i>=0; i--)
		iwt_block(wl_image->v, buf, wl_image->wl_level_v[i].height,
			wl_image->wl_level_v[i].width, wl_image->stride_uv);
}
