#include <stdio.h>
#include <stdlib.h>
#include "libbmp.h"


/* windows bitmap file header */
struct WinBmpHead {
	uint16_t type;
	uint32_t size;
	uint16_t res0;
	uint16_t res1;
	uint32_t offs;
} WinBmpHead;

/* windows bitmap info header */
struct WinBmpInfo {
	uint32_t size;
	uint32_t widt;
	uint32_t heig;
	uint16_t plan;
	uint16_t bitc;
	uint32_t comp;
	uint32_t sizi;
	uint32_t xppm;
	uint32_t yppm;
	uint32_t colo;
	uint32_t coli;
} WinBmpInfo;

/********************************************************************

********************************************************************/
uint32_t BMPLoad(char *fn, int *wi, int *hi, uint8_t *bm) {
	FILE *f;

	if((f = fopen(fn, "rb")) == NULL) {
		printf("Can't open file >%s<", fn);
		return 1;
   }
      
	/* windows bitmap file head */
	fread(&WinBmpHead.type, 1, 2, f);
	fread(&WinBmpHead.size, 1, 4, f);
	fread(&WinBmpHead.res0, 1, 2, f);
	fread(&WinBmpHead.res1, 1, 2, f);
	fread(&WinBmpHead.offs, 1, 4, f);
   
	/* windows bitmap info head */
	fread(&WinBmpInfo.size, 1, 4, f);
	fread(&WinBmpInfo.widt, 1, 4, f);
	fread(&WinBmpInfo.heig, 1, 4, f);
	fread(&WinBmpInfo.plan, 1, 2, f);
	fread(&WinBmpInfo.bitc, 1, 2, f);
	fread(&WinBmpInfo.comp, 1, 4, f);
	fread(&WinBmpInfo.sizi, 1, 4, f);
	fread(&WinBmpInfo.xppm, 1, 4, f);
	fread(&WinBmpInfo.yppm, 1, 4, f);
	fread(&WinBmpInfo.colo, 1, 4, f);
	fread(&WinBmpInfo.coli, 1, 4, f);
   
	if((WinBmpHead.type != 19778) || (WinBmpInfo.bitc != 24)) {
		fclose(f);
		return 1;
	}
      
	if(wi != NULL)
		*wi = WinBmpInfo.widt;
	if(hi != NULL)
		*hi = WinBmpInfo.heig;

	fseek(f, WinBmpHead.offs, 0);
   
	fread(bm, WinBmpInfo.widt*WinBmpInfo.heig*3, 1, f);
   
	fclose(f);
   
	return 0;
}


/********************************************************************

********************************************************************/
uint32_t BMPInfo(char *fn, int *wi, int *hi) {
	FILE *f;

	if((f = fopen(fn, "rb")) == NULL) {
		printf("Can't open file >%s<", fn);
		return 1;
   }
      
	/* windows bitmap file head */
	fread(&WinBmpHead.type, 1, 2, f);
	fread(&WinBmpHead.size, 1, 4, f);
	fread(&WinBmpHead.res0, 1, 2, f);
	fread(&WinBmpHead.res1, 1, 2, f);
	fread(&WinBmpHead.offs, 1, 4, f);
   
	/* windows bitmap info head */
	fread(&WinBmpInfo.size, 1, 4, f);
	fread(&WinBmpInfo.widt, 1, 4, f);
	fread(&WinBmpInfo.heig, 1, 4, f);
	fread(&WinBmpInfo.plan, 1, 2, f);
	fread(&WinBmpInfo.bitc, 1, 2, f);
	fread(&WinBmpInfo.comp, 1, 4, f);
	fread(&WinBmpInfo.sizi, 1, 4, f);
	fread(&WinBmpInfo.xppm, 1, 4, f);
	fread(&WinBmpInfo.yppm, 1, 4, f);
	fread(&WinBmpInfo.colo, 1, 4, f);
	fread(&WinBmpInfo.coli, 1, 4, f);
   
	if((WinBmpHead.type != 19778) || (WinBmpInfo.bitc != 24)) {
		fclose(f);
		return 1;
	}
   
	if(wi != NULL)
		*wi = WinBmpInfo.widt;
	if(hi != NULL)
		*hi = WinBmpInfo.heig;
   
	fclose(f);
   
	return 0;
}

/********************************************************************

********************************************************************/
uint32_t BMPSave(char *fn, int wi, int hi, uint8_t *bm) {
	FILE *f;

	if((f = fopen(fn, "wb")) == NULL) {
		printf("Can't open file >%s<", fn);
		return 1;
   }

	/* windows bitmap file header */
	WinBmpHead.type = 19778;
	WinBmpHead.size = 54+wi*hi*3;
	WinBmpHead.res0 = 0;
	WinBmpHead.res1 = 0;
	WinBmpHead.offs = 54;

	/* windos bitmap info header */
	WinBmpInfo.size = 40;
	WinBmpInfo.widt = wi;
	WinBmpInfo.heig = hi;
	WinBmpInfo.plan = 1;
	WinBmpInfo.bitc = 24;
	WinBmpInfo.comp = 0;
	WinBmpInfo.sizi = wi*hi*3;
	WinBmpInfo.xppm = 3780;
	WinBmpInfo.yppm = 3780;
	WinBmpInfo.colo = 0;
	WinBmpInfo.coli = 0;
   
	fwrite(&WinBmpHead.type, 1, 2, f);
	fwrite(&WinBmpHead.size, 1, 4, f);
	fwrite(&WinBmpHead.res0, 1, 2, f);
	fwrite(&WinBmpHead.res1, 1, 2, f);
	fwrite(&WinBmpHead.offs, 1, 4, f);
   
	fwrite(&WinBmpInfo.size, 1, 4, f);
	fwrite(&WinBmpInfo.widt, 1, 4, f);
	fwrite(&WinBmpInfo.heig, 1, 4, f);
	fwrite(&WinBmpInfo.plan, 1, 2, f);
	fwrite(&WinBmpInfo.bitc, 1, 2, f);
	fwrite(&WinBmpInfo.comp, 1, 4, f);
	fwrite(&WinBmpInfo.sizi, 1, 4, f);
	fwrite(&WinBmpInfo.xppm, 1, 4, f);
	fwrite(&WinBmpInfo.yppm, 1, 4, f);
	fwrite(&WinBmpInfo.colo, 1, 4, f);
	fwrite(&WinBmpInfo.coli, 1, 4, f);
   
	fwrite(bm, wi*hi*3, 1, f);
   
	fclose(f);
   
	return 0;
}
   
   
