#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* bwave api */
#include "../src/libbw.h"

/* bw_malloc */
#include "../src/global.h"

/* BMP* functions */
#include "libbmp.h"



FILE *encore_file_init(char *filename, BW_ENC_PARAM *param) {
	FILE *file;

	if((file = fopen(filename, "wb")) == NULL) {
		printf("Can't open >%s<\n", filename);
		return NULL;
	}

	fwrite((void*)&param->width, 1, sizeof(int), file);
	fwrite((void*)&param->height, 1, sizeof(int), file);

	return file;
}


void encore_file_fput(FILE *file, BW_ENC_FRAME *frame) {
	if(file) {
		fwrite((void*)&frame->size, 1, sizeof(int), file);
		fwrite((void*)frame->data, frame->size, sizeof(uint8_t), file);
	}
}


void encore_file_stop(FILE *file) {
	if(file)
		fclose(file);
}


FILE *decore_file_init(char *filename, BW_DEC_PARAM *param)	{
	FILE *file;

	if((file = fopen(filename, "rb")) == NULL) {
		printf("Can't open >%s<\n", filename);
		return NULL;
	}

	fread((void*)&param->width, 1, sizeof(int), file);
	fread((void*)&param->height, 1, sizeof(int), file);

	return file;
}


void decore_file_fget(FILE *file, BW_DEC_FRAME *frame) {
	if(file) {
		fread((void*)&frame->size, 1, sizeof(int), file);
		fread((void*)frame->data, frame->size, sizeof(uint8_t), file);
	}
}


void decore_file_stop(FILE *file) {
	if(file)
		fclose(file);
}




void test_encore(char *input, char *bw_file, int quant, int start, int end) {
	int i;
	FILE *file;
	char filename[256];
	BW_ENC_PARAM param;
	BW_ENC_FRAME frame;

	printf("start encoding...\n");
	sprintf(filename, "%s\\%04d.bmp", input, start);
	BMPInfo(filename, &param.width, &param.height);
	frame.data = bw_malloc(param.width*param.height*6);
	frame.image = bw_malloc(param.width*param.height*4);
	frame.stride = param.width*3;
	frame.colorspace = BW_COL_RGB24;
	frame.quant = quant;
	frame.deadz = (frame.quant*3)/2;
	frame.size = param.width*param.height*3;
	bw_encore(NULL, BW_ENC_CREATE, &param);
	file = encore_file_init(bw_file, &param);

	for(i=start; i<=end; i++) {
		sprintf(filename, "%s\\%04d.bmp", input, i);
		BMPLoad(filename, NULL, NULL, frame.image);
		bw_encore(param.handle, BW_ENC_ENCODE, &frame);
		encore_file_fput(file, &frame);
		printf("\rframe: %4d size: %6db", i, frame.size);
	}

	encore_file_stop(file);
	bw_encore(param.handle, BW_ENC_DESTROY, NULL);
	printf("\nstop encoding\n");
	bw_free(frame.image);
	bw_free(frame.data);
}


void test_decore(char *output, char *bw_file, int end) {
	int i;
	FILE *file;
	char filename[256];
	BW_DEC_PARAM param;
	BW_DEC_FRAME frame;

	printf("start decoding...\n");
	file = decore_file_init(bw_file, &param);
	frame.data = bw_malloc(param.width*param.height*6);
	frame.image = bw_malloc(param.width*param.height*3);
	frame.stride = param.width*3;
	frame.colorspace = BW_COL_RGB24;
	bw_decore(NULL, BW_DEC_CREATE, &param);

	for(i=0; i<end; i++) {
		decore_file_fget(file, &frame);
		bw_decore(param.handle, BW_DEC_DECODE, &frame);
		sprintf(filename, "%s\\%04d.bmp", output, i);
		BMPSave(filename, param.width, param.height, frame.image);
		printf("\rframe: %4d size: %6db", i, frame.size);
	}

	decore_file_stop(file);
	bw_decore(param.handle, BW_DEC_DESTROY, NULL);
	printf("\nstop decoding\n");
	bw_free(frame.image);
	bw_free(frame.data);
}


void usage() {
	printf("usage: test mode input/output bw_file [parameters]\n\n");
	printf("	mode: (e)ncode, (d)ecode\n");
	printf("	input/output: in- or output dir filled with xxxx.bmp\n");
	printf("	bw_file: compressed file\n");
	printf("	parameters - encoding: quant start end\n");
	printf("	parameters - decoding: end\n");
}

int main(int argc, char *argv[]) {

	printf("Copyright (C) 2004 by Bergwinkl Thomas\n");
	if(argc < 2) {
		usage();
		return -1;
	}

	if(argv[1][0] == 'e') {
		int quant, start, end;
		
		if(argc != 7) {
			usage();
			return -1;
		}
		
		quant = atoi(argv[4]);
		start = atoi(argv[5]);
		end = atoi(argv[6]);

		printf("encoding parameters:\n");
		printf("input: %s\n", argv[2]);
		printf("bs_file: %s\n", argv[3]);
		printf("quant: %d\n", quant);
		printf("start: %d\n", start);
		printf("end: %d\n", end);

		test_encore(argv[2], argv[3], quant, start, end);

	} else if(argv[1][0] == 'd') {
		int end;

		if(argc != 5) {
			usage();
			return -1;
		}

		end = atoi(argv[4]);

		printf("decoding parameters:\n");
		printf("output: %s\n", argv[2]);
		printf("bs_file: %s\n", argv[3]);
		printf("end: %d\n", end);
		
		test_decore(argv[2], argv[3], end);
	}

	return 0;
}
