#include <stdio.h>
#include <windows.h>
#include "vfw.h"

#include "codec.h"
#include "config.h"
#include "resource.h"



void DEBUG(char *str) {
	FILE *file;
	
	if(config.d_active) {
		file = fopen(config.d_filename, "at");

		if(file != NULL) {
			fputs(str, file);
			fclose(file);
		}
	}
}


void DEBUG_PRINTF(char *str, ...) {
	va_list args;
	char buf[1024];

	va_start(args, str);
	vsprintf(buf, str, args);
	DEBUG(buf);
}


void DEBUG_BITMAPINFOHEADER(BITMAPINFOHEADER *hdr) {
	DEBUG_PRINTF("BITMAPINFOHEADER:\n");
	DEBUG_PRINTF(" biSize: %u\n", hdr->biSize);
	DEBUG_PRINTF(" biWidth: %d\n", hdr->biWidth);
	DEBUG_PRINTF(" biHeight: %d\n", hdr->biHeight);
	DEBUG_PRINTF(" biPlanes: %d\n", hdr->biPlanes);
	DEBUG_PRINTF(" biBitCount: %d\n", hdr->biBitCount);
	DEBUG_PRINTF(" biCompression: %u\n", hdr->biCompression);
	DEBUG_PRINTF(" biSizeImage: %u\n", hdr->biSizeImage);
	DEBUG_PRINTF(" biXPelsPerMeter: %d\n", hdr->biXPelsPerMeter);
	DEBUG_PRINTF(" biYPelsPerMeter: %d\n", hdr->biYPelsPerMeter);
	DEBUG_PRINTF(" biClrUsed: %u\n", hdr->biClrUsed);
	DEBUG_PRINTF(" biClrImportant: %u\n", hdr->biClrImportant);
}


int get_colorspace(BITMAPINFOHEADER *hdr) {
	if((hdr->biWidth&1) || (hdr->biHeight&1))
		return BW_COL_NULL;

	switch(hdr->biCompression) {
		case BI_RGB:      
			switch(hdr->biBitCount) {
				/*case 15:
					return BW_COL_RG15;*/
				/*case 16:
					return BW_COL_RG16;*/
				case 24:
					return BW_COL_RGB24;
				default:
					return BW_COL_NULL;
			}

		case FOURCC_YV12:
			return BW_COL_YV12;

		default:
			return BW_COL_NULL;
	}
}



LRESULT drv_open(ICOPEN *icopen) {
	CODEC *codec;
	
	DEBUG_PRINTF("Calling drv_open()\n");

	if((icopen != NULL) && (icopen->fccType != ICTYPE_VIDEO)) {
		return DRV_CANCEL;
	}

	codec = (CODEC*)malloc(sizeof(CODEC));
	codec->enc_param.handle = NULL;
	codec->dec_param.handle = NULL;

	if(codec == NULL) {
		if(icopen != NULL) {
			icopen->dwError = ICERR_MEMORY;
		}

		return 0;
	}

	if(icopen != NULL) {
		icopen->dwError = ICERR_OK;
	}

	get_config();

	return (LRESULT)codec;
}

LRESULT drv_close(CODEC *codec) {
	DEBUG_PRINTF("Calling drv_close()\n");

	free(codec);

	return DRV_OK;
}


LRESULT icm_getinfo(ICINFO *icinfo, LRESULT lParam) {
	DEBUG_PRINTF("Calling icm_getinfo()\n");

	icinfo->fccType = ICTYPE_VIDEO;
	icinfo->fccHandler = FOURCC_BW00;
	icinfo->dwFlags = VIDCF_FASTTEMPORALC | VIDCF_FASTTEMPORALD | VIDCF_COMPRESSFRAMES;
	icinfo->dwVersion = 0;
	icinfo->dwVersionICM = ICVERSION;

	wcscpy(icinfo->szName, BW00_NAME_L); 
	wcscpy(icinfo->szDescription, BW00_DESC_L);

	return lParam;
}


LRESULT icm_about(HWND hwnd) {
	DEBUG_PRINTF("Calling icm_about()\n");

	DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_ABOUT), (HWND)hwnd, AboutProc, 0);

	return ICERR_OK;
}


LRESULT icm_configure(HWND hwnd) {
	DEBUG_PRINTF("Calling icm_configure()\n");

	DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_CONFIG), (HWND)hwnd, ConfigProc, 0);

	return ICERR_OK;
}


LRESULT icm_compress_query(CODEC *codec, BITMAPINFO *lpbiInput, BITMAPINFO *lpbiOutput) {
	DEBUG_PRINTF("Calling icm_compress_query()\n");

	if(get_colorspace(&lpbiInput->bmiHeader) == BW_COL_NULL) {
		return ICERR_BADFORMAT;
	}

	if(lpbiOutput == NULL) {
		return ICERR_OK;
	}

	if((lpbiInput->bmiHeader.biWidth != lpbiOutput->bmiHeader.biWidth) || (lpbiInput->bmiHeader.biHeight != lpbiOutput->bmiHeader.biHeight) || (lpbiOutput->bmiHeader.biHeight != FOURCC_BW00)) {
		return ICERR_BADFORMAT;
	}

	return ICERR_OK;
}


LRESULT icm_compress_get_format(CODEC *codec, BITMAPINFO *lpbiInput, BITMAPINFO *lpbiOutput) {
	DEBUG_PRINTF("Calling icm_compress_get_format()\n");

	if(get_colorspace(&lpbiInput->bmiHeader) == BW_COL_NULL) {
		return ICERR_BADFORMAT;
	}

	if (lpbiOutput == NULL) {
		return sizeof(BITMAPINFOHEADER);
	}

	lpbiOutput->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	lpbiOutput->bmiHeader.biWidth = lpbiInput->bmiHeader.biWidth;
	lpbiOutput->bmiHeader.biHeight = lpbiInput->bmiHeader.biHeight;
	lpbiOutput->bmiHeader.biPlanes = 1;
	lpbiOutput->bmiHeader.biBitCount = 12;
	lpbiOutput->bmiHeader.biCompression = FOURCC_BW00;
	lpbiOutput->bmiHeader.biSizeImage = icm_compress_get_size(codec, lpbiInput, lpbiOutput);
	lpbiOutput->bmiHeader.biXPelsPerMeter = 0;
	lpbiOutput->bmiHeader.biYPelsPerMeter = 0;
	lpbiOutput->bmiHeader.biClrUsed = 0;
	lpbiOutput->bmiHeader.biClrImportant = 0;

	return ICERR_OK;
}


LRESULT icm_compress_get_size(CODEC *codec, BITMAPINFO *lpbiInput, BITMAPINFO *lpbiOutput) {
	DEBUG_PRINTF("Calling icm_compress_get_size()\n");

	return (lpbiOutput->bmiHeader.biWidth*lpbiOutput->bmiHeader.biHeight*6);
}


LRESULT icm_compress_frames_info(CODEC *codec, ICCOMPRESSFRAMES *icf) {
	DEBUG_PRINTF("Calling icm_compress_frames_info()\n");

	return ICERR_OK;
}


LRESULT icm_compress_begin(CODEC *codec, BITMAPINFO *lpbiInput, BITMAPINFO *lpbiOutput) {
	DEBUG_PRINTF("Calling icm_compress_begin()\n");

	if(codec->enc_param.handle) {
		bw_encore(codec->enc_param.handle, BW_ENC_DESTROY, NULL);
	}

	codec->enc_param.width = lpbiInput->bmiHeader.biWidth;
	codec->enc_param.height = lpbiInput->bmiHeader.biHeight;   
	bw_encore(NULL, BW_ENC_CREATE, &codec->enc_param);

	return ICERR_OK;
}


LRESULT icm_compress_end(CODEC *codec) {
	DEBUG_PRINTF("Calling icm_compress_end()\n");

	if(codec->enc_param.handle) {
		bw_encore(codec->enc_param.handle, BW_ENC_DESTROY, NULL);
		codec->enc_param.handle = NULL;
	}

	return ICERR_OK;
}


LRESULT icm_compress(CODEC *codec, ICCOMPRESS *icc) {
	BW_ENC_FRAME frame;

	DEBUG_PRINTF("Calling icm_compress()\n");

	if((frame.colorspace = get_colorspace(icc->lpbiInput)) == BW_COL_NULL) {
		return ICERR_BADFORMAT;
	}

	frame.data = icc->lpOutput;
	frame.size = icc->lpbiOutput->biSizeImage;

	frame.image = icc->lpInput;
	frame.quant = config.i_quant;
	frame.deadz = config.i_deadz;
	/*frame.fk = 0;*/

	switch(frame.colorspace) {
		case BW_COL_RGB24:
			frame.stride = icc->lpbiOutput->biWidth*3;
			break;

		case BW_COL_YV12:
			frame.stride = icc->lpbiOutput->biWidth;
			break;
	}

	if(frame.colorspace == BW_COL_NULL) {
		return ICERR_BADFORMAT;
	}

	bw_encore(codec->enc_param.handle, BW_ENC_ENCODE, &frame);

	/*if(frame.ty)
		*icc->lpdwFlags = 0;
	else*/
		*icc->lpdwFlags = AVIIF_KEYFRAME;

	icc->lpbiOutput->biSizeImage = frame.size;

	return ICERR_OK;
}


LRESULT icm_decompress_query(CODEC *codec, BITMAPINFO *lpbiInput, BITMAPINFO *lpbiOutput) {
	DEBUG_PRINTF("Calling icm_decompress_query()\n");

	if(lpbiInput == NULL) {
		return ICERR_ERROR;
	}

	if(lpbiInput->bmiHeader.biCompression != FOURCC_BW00) {
		return ICERR_BADFORMAT;
	}

	if(lpbiOutput == NULL) {
		return ICERR_OK;
	}

	if((lpbiInput->bmiHeader.biWidth != lpbiOutput->bmiHeader.biWidth) || (lpbiInput->bmiHeader.biHeight != lpbiOutput->bmiHeader.biHeight) || (get_colorspace(&lpbiOutput->bmiHeader) == BW_COL_NULL)) {
		return ICERR_BADFORMAT;
	}

	return ICERR_OK;
}


LRESULT icm_decompress_get_format(CODEC *codec, BITMAPINFO *lpbiInput, BITMAPINFO *lpbiOutput) {
	DEBUG_PRINTF("Calling icm_decompress_get_format()\n");

	if(lpbiOutput == NULL) {
		return sizeof(BITMAPINFOHEADER);
	}

	if(lpbiInput == NULL) {
		return ICERR_ERROR;
	}

	if(lpbiInput->bmiHeader.biCompression != FOURCC_BW00) {
		return ICERR_BADFORMAT;
	}

	lpbiOutput->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	lpbiOutput->bmiHeader.biWidth = lpbiInput->bmiHeader.biWidth;
	lpbiOutput->bmiHeader.biHeight = lpbiInput->bmiHeader.biHeight;
	lpbiOutput->bmiHeader.biPlanes = 1;
	lpbiOutput->bmiHeader.biBitCount = 24;
	lpbiOutput->bmiHeader.biCompression = BI_RGB;
	lpbiOutput->bmiHeader.biSizeImage = lpbiOutput->bmiHeader.biWidth * lpbiOutput->bmiHeader.biHeight * lpbiOutput->bmiHeader.biBitCount/8;
	lpbiOutput->bmiHeader.biXPelsPerMeter = 0;
	lpbiOutput->bmiHeader.biYPelsPerMeter = 0;
	lpbiOutput->bmiHeader.biClrUsed = 0;
	lpbiOutput->bmiHeader.biClrImportant = 0;

	return ICERR_OK;	
}


LRESULT icm_decompress_begin(CODEC *codec, BITMAPINFO *lpbiInput, BITMAPINFO *lpbiOutput) {	
	DEBUG_PRINTF("Calling icm_decompress_begin();\n");

	if((lpbiInput->bmiHeader.biWidth != lpbiOutput->bmiHeader.biWidth) || (lpbiInput->bmiHeader.biHeight != lpbiOutput->bmiHeader.biHeight) || (get_colorspace(&lpbiOutput->bmiHeader) == BW_COL_NULL)) {
		return ICERR_BADFORMAT;
	}

	codec->dec_param.width = lpbiInput->bmiHeader.biWidth;
	codec->dec_param.height = lpbiInput->bmiHeader.biHeight;

	bw_decore(NULL, BW_DEC_CREATE, &codec->dec_param);

	return ICERR_OK;
}


LRESULT icm_decompress_end(CODEC *codec) {
	DEBUG_PRINTF("Calling icm_decompress_end()\n");

	if(codec->dec_param.handle != NULL) {
		bw_decore(codec->dec_param.handle, BW_DEC_DESTROY, NULL);
		codec->dec_param.handle = NULL;
	}

	return ICERR_OK;
}


LRESULT icm_decompress(CODEC *codec, ICDECOMPRESS *icd) {
	BW_DEC_FRAME frame;

	DEBUG_PRINTF("Calling icm_decompress()\n");

	if((frame.colorspace = get_colorspace(icd->lpbiOutput)) == BW_COL_NULL) {
		return ICERR_BADFORMAT;
	}

	frame.data = icd->lpInput;
	frame.size = icd->lpbiInput->biSizeImage;
	frame.image = icd->lpOutput;
	
	switch(frame.colorspace) {
		case BW_COL_RGB24:
			frame.stride = icd->lpbiOutput->biWidth*3;
			break;

		case BW_COL_YV12:
			frame.stride = icd->lpbiOutput->biWidth;
			break;
	}

	bw_decore(codec->dec_param.handle, BW_DEC_DECODE, &frame);

	return ICERR_OK;
}
